""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/address6
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class Address6MacaddrItem:
    """Nested item for macaddr field - supports attribute access."""
    macaddr: str


class Address6TaggingItem:
    """Nested item for tagging field - supports attribute access."""
    name: str
    category: str
    tags: str


class Address6SubnetsegmentItem:
    """Nested item for subnet-segment field - supports attribute access."""
    name: str
    type: Literal["any", "specific"]
    value: str


class Address6ListItem:
    """Nested item for list field - supports attribute access."""
    ip: str


class Address6Payload(TypedDict, total=False):
    """Payload type for Address6 operations."""
    name: str
    uuid: str
    type: Literal["ipprefix", "iprange", "fqdn", "geography", "dynamic", "template", "mac", "route-tag", "wildcard"]
    route_tag: int
    macaddr: str | list[str] | list[dict[str, Any]] | list[Address6MacaddrItem]
    sdn: str
    ip6: str
    wildcard: str
    start_ip: str
    end_ip: str
    fqdn: str
    country: str
    cache_ttl: int
    color: int
    obj_id: str
    tagging: str | list[str] | list[dict[str, Any]] | list[Address6TaggingItem]
    comment: str
    template: str
    subnet_segment: str | list[str] | list[dict[str, Any]] | list[Address6SubnetsegmentItem]
    host_type: Literal["any", "specific"]
    host: str
    tenant: str
    epg_name: str
    sdn_tag: str
    filter: str
    list: str | list[str] | list[dict[str, Any]] | list[Address6ListItem]
    sdn_addr_type: Literal["private", "public", "all"]
    passive_fqdn_learning: Literal["disable", "enable"]
    fabric_object: Literal["enable", "disable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class Address6Response(TypedDict, total=False):
    """Response type for Address6 - use with .dict property for typed dict access."""
    name: str
    uuid: str
    type: Literal["ipprefix", "iprange", "fqdn", "geography", "dynamic", "template", "mac", "route-tag", "wildcard"]
    route_tag: int
    macaddr: list[Address6MacaddrItem]
    sdn: str
    ip6: str
    wildcard: str
    start_ip: str
    end_ip: str
    fqdn: str
    country: str
    cache_ttl: int
    color: int
    obj_id: str
    tagging: list[Address6TaggingItem]
    comment: str
    template: str
    subnet_segment: list[Address6SubnetsegmentItem]
    host_type: Literal["any", "specific"]
    host: str
    tenant: str
    epg_name: str
    sdn_tag: str
    filter: str
    list: list[Address6ListItem]
    sdn_addr_type: Literal["private", "public", "all"]
    passive_fqdn_learning: Literal["disable", "enable"]
    fabric_object: Literal["enable", "disable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class Address6Object(FortiObject):
    """Typed FortiObject for Address6 with field access."""
    name: str
    uuid: str
    type: Literal["ipprefix", "iprange", "fqdn", "geography", "dynamic", "template", "mac", "route-tag", "wildcard"]
    route_tag: int
    macaddr: list[Address6MacaddrItem]
    sdn: str
    ip6: str
    wildcard: str
    start_ip: str
    end_ip: str
    fqdn: str
    country: str
    cache_ttl: int
    color: int
    obj_id: str
    tagging: list[Address6TaggingItem]
    comment: str
    template: str
    subnet_segment: list[Address6SubnetsegmentItem]
    host_type: Literal["any", "specific"]
    host: str
    tenant: str
    epg_name: str
    sdn_tag: str
    filter: str
    sdn_addr_type: Literal["private", "public", "all"]
    passive_fqdn_learning: Literal["disable", "enable"]
    fabric_object: Literal["enable", "disable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Address6:
    """
    
    Endpoint: firewall/address6
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Address6Object: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[Address6Object]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: Address6Payload | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        type: Literal["ipprefix", "iprange", "fqdn", "geography", "dynamic", "template", "mac", "route-tag", "wildcard"] | None = ...,
        route_tag: int | None = ...,
        macaddr: str | list[str] | list[dict[str, Any]] | list[Address6MacaddrItem] | None = ...,
        sdn: str | None = ...,
        ip6: str | None = ...,
        wildcard: str | None = ...,
        start_ip: str | None = ...,
        end_ip: str | None = ...,
        fqdn: str | None = ...,
        country: str | None = ...,
        cache_ttl: int | None = ...,
        color: int | None = ...,
        obj_id: str | None = ...,
        tagging: str | list[str] | list[dict[str, Any]] | list[Address6TaggingItem] | None = ...,
        comment: str | None = ...,
        template: str | None = ...,
        subnet_segment: str | list[str] | list[dict[str, Any]] | list[Address6SubnetsegmentItem] | None = ...,
        host_type: Literal["any", "specific"] | None = ...,
        host: str | None = ...,
        tenant: str | None = ...,
        epg_name: str | None = ...,
        sdn_tag: str | None = ...,
        filter: str | None = ...,
        list: str | list[str] | list[dict[str, Any]] | list[Address6ListItem] | None = ...,
        sdn_addr_type: Literal["private", "public", "all"] | None = ...,
        passive_fqdn_learning: Literal["disable", "enable"] | None = ...,
        fabric_object: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Address6Object: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: Address6Payload | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        type: Literal["ipprefix", "iprange", "fqdn", "geography", "dynamic", "template", "mac", "route-tag", "wildcard"] | None = ...,
        route_tag: int | None = ...,
        macaddr: str | list[str] | list[dict[str, Any]] | list[Address6MacaddrItem] | None = ...,
        sdn: str | None = ...,
        ip6: str | None = ...,
        wildcard: str | None = ...,
        start_ip: str | None = ...,
        end_ip: str | None = ...,
        fqdn: str | None = ...,
        country: str | None = ...,
        cache_ttl: int | None = ...,
        color: int | None = ...,
        obj_id: str | None = ...,
        tagging: str | list[str] | list[dict[str, Any]] | list[Address6TaggingItem] | None = ...,
        comment: str | None = ...,
        template: str | None = ...,
        subnet_segment: str | list[str] | list[dict[str, Any]] | list[Address6SubnetsegmentItem] | None = ...,
        host_type: Literal["any", "specific"] | None = ...,
        host: str | None = ...,
        tenant: str | None = ...,
        epg_name: str | None = ...,
        sdn_tag: str | None = ...,
        filter: str | None = ...,
        list: str | list[str] | list[dict[str, Any]] | list[Address6ListItem] | None = ...,
        sdn_addr_type: Literal["private", "public", "all"] | None = ...,
        passive_fqdn_learning: Literal["disable", "enable"] | None = ...,
        fabric_object: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> Address6Object: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: Address6Payload | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        type: Literal["ipprefix", "iprange", "fqdn", "geography", "dynamic", "template", "mac", "route-tag", "wildcard"] | None = ...,
        route_tag: int | None = ...,
        macaddr: str | list[str] | list[dict[str, Any]] | list[Address6MacaddrItem] | None = ...,
        sdn: str | None = ...,
        ip6: str | None = ...,
        wildcard: str | None = ...,
        start_ip: str | None = ...,
        end_ip: str | None = ...,
        fqdn: str | None = ...,
        country: str | None = ...,
        cache_ttl: int | None = ...,
        color: int | None = ...,
        obj_id: str | None = ...,
        tagging: str | list[str] | list[dict[str, Any]] | list[Address6TaggingItem] | None = ...,
        comment: str | None = ...,
        template: str | None = ...,
        subnet_segment: str | list[str] | list[dict[str, Any]] | list[Address6SubnetsegmentItem] | None = ...,
        host_type: Literal["any", "specific"] | None = ...,
        host: str | None = ...,
        tenant: str | None = ...,
        epg_name: str | None = ...,
        sdn_tag: str | None = ...,
        filter: str | None = ...,
        list: str | list[str] | list[dict[str, Any]] | list[Address6ListItem] | None = ...,
        sdn_addr_type: Literal["private", "public", "all"] | None = ...,
        passive_fqdn_learning: Literal["disable", "enable"] | None = ...,
        fabric_object: Literal["enable", "disable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Address6",
    "Address6Payload",
    "Address6Response",
    "Address6Object",
]


"""FortiOS CMDB - FileFilter category"""

from .profile import Profile

__all__ = [
    "FileFilter",
    "Profile",
]


class FileFilter:
    """FileFilter endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """FileFilter endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.profile = Profile(client)



"""
Pydantic Models for CMDB - log/fortianalyzer2/filter

Runtime validation models for log/fortianalyzer2/filter configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class FilterFreeStyleCategoryEnum(str, Enum):
    """Allowed values for category field in free-style."""
    TRAFFIC = "traffic"
    EVENT = "event"
    VIRUS = "virus"
    WEBFILTER = "webfilter"
    ATTACK = "attack"
    SPAM = "spam"
    ANOMALY = "anomaly"
    VOIP = "voip"
    DLP = "dlp"
    APP_CTRL = "app-ctrl"
    WAF = "waf"
    GTP = "gtp"
    DNS = "dns"
    SSH = "ssh"
    SSL = "ssl"
    FILE_FILTER = "file-filter"
    ICAP = "icap"
    VIRTUAL_PATCH = "virtual-patch"
    DEBUG = "debug"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class FilterFreeStyle(BaseModel):
    """
    Child table model for free-style.
    
    Free style filters.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Entry ID.")    
    category: FilterFreeStyleCategoryEnum = Field(default=FilterFreeStyleCategoryEnum.TRAFFIC, description="Log category.")    
    filter_: str = Field(max_length=1023, serialization_alias="filter", description="Free style filter string.")    
    filter_type: Literal["include", "exclude"] | None = Field(default="include", description="Include/exclude logs that match the filter.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class FilterSeverityEnum(str, Enum):
    """Allowed values for severity field."""
    EMERGENCY = "emergency"
    ALERT = "alert"
    CRITICAL = "critical"
    ERROR = "error"
    WARNING = "warning"
    NOTIFICATION = "notification"
    INFORMATION = "information"
    DEBUG = "debug"


# ============================================================================
# Main Model
# ============================================================================

class FilterModel(BaseModel):
    """
    Pydantic model for log/fortianalyzer2/filter configuration.
    
    Filters for FortiAnalyzer.
    
    Validation Rules:        - severity: pattern=        - forward_traffic: pattern=        - local_traffic: pattern=        - multicast_traffic: pattern=        - sniffer_traffic: pattern=        - ztna_traffic: pattern=        - http_transaction: pattern=        - anomaly: pattern=        - voip: pattern=        - dlp_archive: pattern=        - gtp: pattern=        - forti_switch: pattern=        - free_style: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    severity: FilterSeverityEnum | None = Field(default=FilterSeverityEnum.INFORMATION, description="Log every message above and including this severity level.")    
    forward_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable forward traffic logging.")    
    local_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable local in or out traffic logging.")    
    multicast_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable multicast traffic logging.")    
    sniffer_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable sniffer traffic logging.")    
    ztna_traffic: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable ztna traffic logging.")    
    http_transaction: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable log HTTP transaction messages.")    
    anomaly: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable anomaly logging.")    
    voip: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable VoIP logging.")    
    dlp_archive: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable DLP archive logging.")    
    gtp: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable GTP messages logging.")    
    forti_switch: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Forti-Switch logging.")    
    free_style: list[FilterFreeStyle] = Field(default_factory=list, description="Free style filters.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "FilterModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "FilterModel",    "FilterFreeStyle",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.270508Z
# ============================================================================


"""Validation helpers for log/syslogd4/filter - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "severity": "information",
    "forward-traffic": "enable",
    "local-traffic": "enable",
    "multicast-traffic": "enable",
    "sniffer-traffic": "enable",
    "ztna-traffic": "enable",
    "http-transaction": "enable",
    "anomaly": "enable",
    "voip": "enable",
    "gtp": "enable",
    "forti-switch": "enable",
    "debug": "disable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "severity": "option",  # Lowest severity level to log.
    "forward-traffic": "option",  # Enable/disable forward traffic logging.
    "local-traffic": "option",  # Enable/disable local in or out traffic logging.
    "multicast-traffic": "option",  # Enable/disable multicast traffic logging.
    "sniffer-traffic": "option",  # Enable/disable sniffer traffic logging.
    "ztna-traffic": "option",  # Enable/disable ztna traffic logging.
    "http-transaction": "option",  # Enable/disable log HTTP transaction messages.
    "anomaly": "option",  # Enable/disable anomaly logging.
    "voip": "option",  # Enable/disable VoIP logging.
    "gtp": "option",  # Enable/disable GTP messages logging.
    "forti-switch": "option",  # Enable/disable Forti-Switch logging.
    "debug": "option",  # Enable/disable debug logging.
    "free-style": "string",  # Free style filters.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "severity": "Lowest severity level to log.",
    "forward-traffic": "Enable/disable forward traffic logging.",
    "local-traffic": "Enable/disable local in or out traffic logging.",
    "multicast-traffic": "Enable/disable multicast traffic logging.",
    "sniffer-traffic": "Enable/disable sniffer traffic logging.",
    "ztna-traffic": "Enable/disable ztna traffic logging.",
    "http-transaction": "Enable/disable log HTTP transaction messages.",
    "anomaly": "Enable/disable anomaly logging.",
    "voip": "Enable/disable VoIP logging.",
    "gtp": "Enable/disable GTP messages logging.",
    "forti-switch": "Enable/disable Forti-Switch logging.",
    "debug": "Enable/disable debug logging.",
    "free-style": "Free style filters.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "free-style": {
        "id": {
            "type": "integer",
            "help": "Entry ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "category": {
            "type": "option",
            "help": "Log category.",
            "required": True,
            "default": "traffic",
            "options": ["traffic", "event", "virus", "webfilter", "attack", "spam", "anomaly", "voip", "dlp", "app-ctrl", "waf", "gtp", "dns", "ssh", "ssl", "file-filter", "icap", "virtual-patch", "debug"],
        },
        "filter": {
            "type": "string",
            "help": "Free style filter string.",
            "required": True,
            "default": "",
            "max_length": 1023,
        },
        "filter-type": {
            "type": "option",
            "help": "Include/exclude logs that match the filter.",
            "default": "include",
            "options": ["include", "exclude"],
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_SEVERITY = [
    "emergency",
    "alert",
    "critical",
    "error",
    "warning",
    "notification",
    "information",
    "debug",
]
VALID_BODY_FORWARD_TRAFFIC = [
    "enable",
    "disable",
]
VALID_BODY_LOCAL_TRAFFIC = [
    "enable",
    "disable",
]
VALID_BODY_MULTICAST_TRAFFIC = [
    "enable",
    "disable",
]
VALID_BODY_SNIFFER_TRAFFIC = [
    "enable",
    "disable",
]
VALID_BODY_ZTNA_TRAFFIC = [
    "enable",
    "disable",
]
VALID_BODY_HTTP_TRANSACTION = [
    "enable",
    "disable",
]
VALID_BODY_ANOMALY = [
    "enable",
    "disable",
]
VALID_BODY_VOIP = [
    "enable",
    "disable",
]
VALID_BODY_GTP = [
    "enable",
    "disable",
]
VALID_BODY_FORTI_SWITCH = [
    "enable",
    "disable",
]
VALID_BODY_DEBUG = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_log_syslogd4_filter_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for log/syslogd4/filter."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_log_syslogd4_filter_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new log/syslogd4/filter object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "severity" in payload:
        is_valid, error = _validate_enum_field(
            "severity",
            payload["severity"],
            VALID_BODY_SEVERITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "forward-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "forward-traffic",
            payload["forward-traffic"],
            VALID_BODY_FORWARD_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "local-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "local-traffic",
            payload["local-traffic"],
            VALID_BODY_LOCAL_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "multicast-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "multicast-traffic",
            payload["multicast-traffic"],
            VALID_BODY_MULTICAST_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "sniffer-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "sniffer-traffic",
            payload["sniffer-traffic"],
            VALID_BODY_SNIFFER_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ztna-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "ztna-traffic",
            payload["ztna-traffic"],
            VALID_BODY_ZTNA_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-transaction" in payload:
        is_valid, error = _validate_enum_field(
            "http-transaction",
            payload["http-transaction"],
            VALID_BODY_HTTP_TRANSACTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "anomaly" in payload:
        is_valid, error = _validate_enum_field(
            "anomaly",
            payload["anomaly"],
            VALID_BODY_ANOMALY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "voip" in payload:
        is_valid, error = _validate_enum_field(
            "voip",
            payload["voip"],
            VALID_BODY_VOIP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "gtp" in payload:
        is_valid, error = _validate_enum_field(
            "gtp",
            payload["gtp"],
            VALID_BODY_GTP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "forti-switch" in payload:
        is_valid, error = _validate_enum_field(
            "forti-switch",
            payload["forti-switch"],
            VALID_BODY_FORTI_SWITCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "debug" in payload:
        is_valid, error = _validate_enum_field(
            "debug",
            payload["debug"],
            VALID_BODY_DEBUG,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_log_syslogd4_filter_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update log/syslogd4/filter."""
    # Validate enum values using central function
    if "severity" in payload:
        is_valid, error = _validate_enum_field(
            "severity",
            payload["severity"],
            VALID_BODY_SEVERITY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "forward-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "forward-traffic",
            payload["forward-traffic"],
            VALID_BODY_FORWARD_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "local-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "local-traffic",
            payload["local-traffic"],
            VALID_BODY_LOCAL_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "multicast-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "multicast-traffic",
            payload["multicast-traffic"],
            VALID_BODY_MULTICAST_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "sniffer-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "sniffer-traffic",
            payload["sniffer-traffic"],
            VALID_BODY_SNIFFER_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ztna-traffic" in payload:
        is_valid, error = _validate_enum_field(
            "ztna-traffic",
            payload["ztna-traffic"],
            VALID_BODY_ZTNA_TRAFFIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "http-transaction" in payload:
        is_valid, error = _validate_enum_field(
            "http-transaction",
            payload["http-transaction"],
            VALID_BODY_HTTP_TRANSACTION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "anomaly" in payload:
        is_valid, error = _validate_enum_field(
            "anomaly",
            payload["anomaly"],
            VALID_BODY_ANOMALY,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "voip" in payload:
        is_valid, error = _validate_enum_field(
            "voip",
            payload["voip"],
            VALID_BODY_VOIP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "gtp" in payload:
        is_valid, error = _validate_enum_field(
            "gtp",
            payload["gtp"],
            VALID_BODY_GTP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "forti-switch" in payload:
        is_valid, error = _validate_enum_field(
            "forti-switch",
            payload["forti-switch"],
            VALID_BODY_FORTI_SWITCH,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "debug" in payload:
        is_valid, error = _validate_enum_field(
            "debug",
            payload["debug"],
            VALID_BODY_DEBUG,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "log/syslogd4/filter",
    "category": "cmdb",
    "api_path": "log.syslogd4/filter",
    "help": "Filters for remote system server.",
    "total_fields": 13,
    "required_fields_count": 0,
    "fields_with_defaults_count": 12,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""Validation helpers for wireless_controller/global_ - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "location": "",
    "acd-process-count": 0,
    "wpad-process-count": 0,
    "image-download": "enable",
    "rolling-wtp-upgrade": "disable",
    "rolling-wtp-upgrade-threshold": "-80",
    "max-retransmit": 3,
    "control-message-offload": "ebp-frame aeroscout-tag ap-list sta-list sta-cap-list stats aeroscout-mu sta-health spectral-analysis",
    "data-ethernet-II": "enable",
    "link-aggregation": "disable",
    "mesh-eth-type": 8755,
    "fiapp-eth-type": 5252,
    "discovery-mc-addr": "224.0.1.140",
    "discovery-mc-addr6": "ff02::18c",
    "max-clients": 0,
    "rogue-scan-mac-adjacency": 7,
    "ipsec-base-ip": "169.254.0.1",
    "wtp-share": "disable",
    "tunnel-mode": "compatible",
    "nac-interval": 120,
    "ap-log-server": "disable",
    "ap-log-server-ip": "0.0.0.0",
    "ap-log-server-port": 0,
    "max-sta-offline": 0,
    "max-sta-offline-ip2mac": 0,
    "max-sta-cap": 0,
    "max-sta-cap-wtp": 8,
    "max-rogue-ap": 0,
    "max-rogue-ap-wtp": 16,
    "max-rogue-sta": 0,
    "max-wids-entry": 0,
    "max-ble-device": 0,
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Name of the wireless controller.
    "location": "string",  # Description of the location of the wireless controller.
    "acd-process-count": "integer",  # Configure the number cw_acd daemons for multi-core CPU suppo
    "wpad-process-count": "integer",  # Wpad daemon process count for multi-core CPU support.
    "image-download": "option",  # Enable/disable WTP image download at join time.
    "rolling-wtp-upgrade": "option",  # Enable/disable rolling WTP upgrade (default = disable).
    "rolling-wtp-upgrade-threshold": "string",  # Minimum signal level/threshold in dBm required for the manag
    "max-retransmit": "integer",  # Maximum number of tunnel packet retransmissions (0 - 64, def
    "control-message-offload": "option",  # Configure CAPWAP control message data channel offload.
    "data-ethernet-II": "option",  # Configure the wireless controller to use Ethernet II or 802.
    "link-aggregation": "option",  # Enable/disable calculating the CAPWAP transmit hash to load 
    "mesh-eth-type": "integer",  # Mesh Ethernet identifier included in backhaul packets (0 - 6
    "fiapp-eth-type": "integer",  # Ethernet type for Fortinet Inter-Access Point Protocol (IAPP
    "discovery-mc-addr": "ipv4-address-multicast",  # Multicast IP address for AP discovery (default = 244.0.1.140
    "discovery-mc-addr6": "ipv6-address",  # Multicast IPv6 address for AP discovery (default = FF02::18C
    "max-clients": "integer",  # Maximum number of clients that can connect simultaneously (d
    "rogue-scan-mac-adjacency": "integer",  # Maximum numerical difference between an AP's Ethernet and wi
    "ipsec-base-ip": "ipv4-address",  # Base IP address for IPsec VPN tunnels between the access poi
    "wtp-share": "option",  # Enable/disable sharing of WTPs between VDOMs.
    "tunnel-mode": "option",  # Compatible/strict tunnel mode.
    "nac-interval": "integer",  # Interval in seconds between two WiFi network access control 
    "ap-log-server": "option",  # Enable/disable configuring FortiGate to redirect wireless ev
    "ap-log-server-ip": "ipv4-address",  # IP address that FortiGate or FortiAPs send log messages to.
    "ap-log-server-port": "integer",  # Port that FortiGate or FortiAPs send log messages to.
    "max-sta-offline": "integer",  # Maximum number of station offline stored on the controller (
    "max-sta-offline-ip2mac": "integer",  # Maximum number of station offline ip2mac stored on the contr
    "max-sta-cap": "integer",  # Maximum number of station cap stored on the controller (defa
    "max-sta-cap-wtp": "integer",  # Maximum number of station cap's wtp info stored on the contr
    "max-rogue-ap": "integer",  # Maximum number of rogue APs stored on the controller (defaul
    "max-rogue-ap-wtp": "integer",  # Maximum number of rogue AP's wtp info stored on the controll
    "max-rogue-sta": "integer",  # Maximum number of rogue stations stored on the controller (d
    "max-wids-entry": "integer",  # Maximum number of wids entries stored on the controller (def
    "max-ble-device": "integer",  # Maximum number of BLE devices stored on the controller (defa
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Name of the wireless controller.",
    "location": "Description of the location of the wireless controller.",
    "acd-process-count": "Configure the number cw_acd daemons for multi-core CPU support (default = 0).",
    "wpad-process-count": "Wpad daemon process count for multi-core CPU support.",
    "image-download": "Enable/disable WTP image download at join time.",
    "rolling-wtp-upgrade": "Enable/disable rolling WTP upgrade (default = disable).",
    "rolling-wtp-upgrade-threshold": "Minimum signal level/threshold in dBm required for the managed WTP to be included in rolling WTP upgrade (-95 to -20, default = -80).",
    "max-retransmit": "Maximum number of tunnel packet retransmissions (0 - 64, default = 3).",
    "control-message-offload": "Configure CAPWAP control message data channel offload.",
    "data-ethernet-II": "Configure the wireless controller to use Ethernet II or 802.3 frames with 802.3 data tunnel mode (default = enable).",
    "link-aggregation": "Enable/disable calculating the CAPWAP transmit hash to load balance sessions to link aggregation nodes (default = disable).",
    "mesh-eth-type": "Mesh Ethernet identifier included in backhaul packets (0 - 65535, default = 8755).",
    "fiapp-eth-type": "Ethernet type for Fortinet Inter-Access Point Protocol (IAPP), or IEEE 802.11f, packets (0 - 65535, default = 5252).",
    "discovery-mc-addr": "Multicast IP address for AP discovery (default = 244.0.1.140).",
    "discovery-mc-addr6": "Multicast IPv6 address for AP discovery (default = FF02::18C).",
    "max-clients": "Maximum number of clients that can connect simultaneously (default = 0, meaning no limitation).",
    "rogue-scan-mac-adjacency": "Maximum numerical difference between an AP's Ethernet and wireless MAC values to match for rogue detection (0 - 31, default = 7).",
    "ipsec-base-ip": "Base IP address for IPsec VPN tunnels between the access points and the wireless controller (default = 169.254.0.1).",
    "wtp-share": "Enable/disable sharing of WTPs between VDOMs.",
    "tunnel-mode": "Compatible/strict tunnel mode.",
    "nac-interval": "Interval in seconds between two WiFi network access control (NAC) checks (10 - 600, default = 120).",
    "ap-log-server": "Enable/disable configuring FortiGate to redirect wireless event log messages or FortiAPs to send UTM log messages to a syslog server (default = disable).",
    "ap-log-server-ip": "IP address that FortiGate or FortiAPs send log messages to.",
    "ap-log-server-port": "Port that FortiGate or FortiAPs send log messages to.",
    "max-sta-offline": "Maximum number of station offline stored on the controller (default = 0).",
    "max-sta-offline-ip2mac": "Maximum number of station offline ip2mac stored on the controller (default = 0).",
    "max-sta-cap": "Maximum number of station cap stored on the controller (default = 0).",
    "max-sta-cap-wtp": "Maximum number of station cap's wtp info stored on the controller (1 - 16, default = 8).",
    "max-rogue-ap": "Maximum number of rogue APs stored on the controller (default = 0).",
    "max-rogue-ap-wtp": "Maximum number of rogue AP's wtp info stored on the controller (1 - 16, default = 16).",
    "max-rogue-sta": "Maximum number of rogue stations stored on the controller (default = 0).",
    "max-wids-entry": "Maximum number of wids entries stored on the controller (default = 0).",
    "max-ble-device": "Maximum number of BLE devices stored on the controller (default = 0).",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 35},
    "location": {"type": "string", "max_length": 35},
    "acd-process-count": {"type": "integer", "min": 0, "max": 255},
    "wpad-process-count": {"type": "integer", "min": 0, "max": 255},
    "rolling-wtp-upgrade-threshold": {"type": "string", "max_length": 7},
    "max-retransmit": {"type": "integer", "min": 0, "max": 64},
    "mesh-eth-type": {"type": "integer", "min": 0, "max": 65535},
    "fiapp-eth-type": {"type": "integer", "min": 0, "max": 65535},
    "max-clients": {"type": "integer", "min": 0, "max": 4294967295},
    "rogue-scan-mac-adjacency": {"type": "integer", "min": 0, "max": 31},
    "nac-interval": {"type": "integer", "min": 10, "max": 600},
    "ap-log-server-port": {"type": "integer", "min": 0, "max": 65535},
    "max-sta-offline": {"type": "integer", "min": 0, "max": 4294967295},
    "max-sta-offline-ip2mac": {"type": "integer", "min": 0, "max": 4294967295},
    "max-sta-cap": {"type": "integer", "min": 0, "max": 4294967295},
    "max-sta-cap-wtp": {"type": "integer", "min": 1, "max": 8},
    "max-rogue-ap": {"type": "integer", "min": 0, "max": 4294967295},
    "max-rogue-ap-wtp": {"type": "integer", "min": 1, "max": 16},
    "max-rogue-sta": {"type": "integer", "min": 0, "max": 4294967295},
    "max-wids-entry": {"type": "integer", "min": 0, "max": 4294967295},
    "max-ble-device": {"type": "integer", "min": 0, "max": 4294967295},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
}


# Valid enum values from API documentation
VALID_BODY_IMAGE_DOWNLOAD = [
    "enable",
    "disable",
]
VALID_BODY_ROLLING_WTP_UPGRADE = [
    "enable",
    "disable",
]
VALID_BODY_CONTROL_MESSAGE_OFFLOAD = [
    "ebp-frame",
    "aeroscout-tag",
    "ap-list",
    "sta-list",
    "sta-cap-list",
    "stats",
    "aeroscout-mu",
    "sta-health",
    "spectral-analysis",
]
VALID_BODY_DATA_ETHERNET_II = [
    "enable",
    "disable",
]
VALID_BODY_LINK_AGGREGATION = [
    "enable",
    "disable",
]
VALID_BODY_WTP_SHARE = [
    "enable",
    "disable",
]
VALID_BODY_TUNNEL_MODE = [
    "compatible",
    "strict",
]
VALID_BODY_AP_LOG_SERVER = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_wireless_controller_global_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for wireless_controller/global_."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_wireless_controller_global_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new wireless_controller/global_ object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "image-download" in payload:
        is_valid, error = _validate_enum_field(
            "image-download",
            payload["image-download"],
            VALID_BODY_IMAGE_DOWNLOAD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "rolling-wtp-upgrade" in payload:
        is_valid, error = _validate_enum_field(
            "rolling-wtp-upgrade",
            payload["rolling-wtp-upgrade"],
            VALID_BODY_ROLLING_WTP_UPGRADE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "control-message-offload" in payload:
        is_valid, error = _validate_enum_field(
            "control-message-offload",
            payload["control-message-offload"],
            VALID_BODY_CONTROL_MESSAGE_OFFLOAD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "data-ethernet-II" in payload:
        is_valid, error = _validate_enum_field(
            "data-ethernet-II",
            payload["data-ethernet-II"],
            VALID_BODY_DATA_ETHERNET_II,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "link-aggregation" in payload:
        is_valid, error = _validate_enum_field(
            "link-aggregation",
            payload["link-aggregation"],
            VALID_BODY_LINK_AGGREGATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "wtp-share" in payload:
        is_valid, error = _validate_enum_field(
            "wtp-share",
            payload["wtp-share"],
            VALID_BODY_WTP_SHARE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "tunnel-mode" in payload:
        is_valid, error = _validate_enum_field(
            "tunnel-mode",
            payload["tunnel-mode"],
            VALID_BODY_TUNNEL_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ap-log-server" in payload:
        is_valid, error = _validate_enum_field(
            "ap-log-server",
            payload["ap-log-server"],
            VALID_BODY_AP_LOG_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_wireless_controller_global_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update wireless_controller/global_."""
    # Validate enum values using central function
    if "image-download" in payload:
        is_valid, error = _validate_enum_field(
            "image-download",
            payload["image-download"],
            VALID_BODY_IMAGE_DOWNLOAD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "rolling-wtp-upgrade" in payload:
        is_valid, error = _validate_enum_field(
            "rolling-wtp-upgrade",
            payload["rolling-wtp-upgrade"],
            VALID_BODY_ROLLING_WTP_UPGRADE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "control-message-offload" in payload:
        is_valid, error = _validate_enum_field(
            "control-message-offload",
            payload["control-message-offload"],
            VALID_BODY_CONTROL_MESSAGE_OFFLOAD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "data-ethernet-II" in payload:
        is_valid, error = _validate_enum_field(
            "data-ethernet-II",
            payload["data-ethernet-II"],
            VALID_BODY_DATA_ETHERNET_II,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "link-aggregation" in payload:
        is_valid, error = _validate_enum_field(
            "link-aggregation",
            payload["link-aggregation"],
            VALID_BODY_LINK_AGGREGATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "wtp-share" in payload:
        is_valid, error = _validate_enum_field(
            "wtp-share",
            payload["wtp-share"],
            VALID_BODY_WTP_SHARE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "tunnel-mode" in payload:
        is_valid, error = _validate_enum_field(
            "tunnel-mode",
            payload["tunnel-mode"],
            VALID_BODY_TUNNEL_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ap-log-server" in payload:
        is_valid, error = _validate_enum_field(
            "ap-log-server",
            payload["ap-log-server"],
            VALID_BODY_AP_LOG_SERVER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "wireless_controller/global_",
    "category": "cmdb",
    "api_path": "wireless-controller/global",
    "help": "Configure wireless controller global settings.",
    "total_fields": 33,
    "required_fields_count": 0,
    "fields_with_defaults_count": 33,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - user/fsso

Runtime validation models for user/fsso configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class FssoModel(BaseModel):
    """
    Pydantic model for user/fsso configuration.
    
    Configure Fortinet Single Sign On (FSSO) agents.
    
    Validation Rules:        - name: max_length=35 pattern=        - type_: pattern=        - server: max_length=63 pattern=        - port: min=1 max=65535 pattern=        - password: max_length=128 pattern=        - server2: max_length=63 pattern=        - port2: min=1 max=65535 pattern=        - password2: max_length=128 pattern=        - server3: max_length=63 pattern=        - port3: min=1 max=65535 pattern=        - password3: max_length=128 pattern=        - server4: max_length=63 pattern=        - port4: min=1 max=65535 pattern=        - password4: max_length=128 pattern=        - server5: max_length=63 pattern=        - port5: min=1 max=65535 pattern=        - password5: max_length=128 pattern=        - logon_timeout: min=1 max=2880 pattern=        - ldap_server: max_length=35 pattern=        - group_poll_interval: min=1 max=2880 pattern=        - ldap_poll: pattern=        - ldap_poll_interval: min=1 max=2880 pattern=        - ldap_poll_filter: max_length=2047 pattern=        - user_info_server: max_length=35 pattern=        - ssl: pattern=        - sni: max_length=255 pattern=        - ssl_server_host_ip_check: pattern=        - ssl_trusted_cert: max_length=79 pattern=        - source_ip: pattern=        - source_ip6: pattern=        - interface_select_method: pattern=        - interface: max_length=15 pattern=        - vrf_select: min=0 max=511 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=35, description="Name.")    
    type_: Literal["default", "fortinac"] | None = Field(default="default", serialization_alias="type", description="Server type.")    
    server: str = Field(max_length=63, description="Domain name or IP address of the first FSSO collector agent.")    
    port: int = Field(ge=1, le=65535, default=8000, description="Port of the first FSSO collector agent.")    
    password: Any = Field(max_length=128, default=None, description="Password of the first FSSO collector agent.")    
    server2: str | None = Field(max_length=63, default=None, description="Domain name or IP address of the second FSSO collector agent.")    
    port2: int | None = Field(ge=1, le=65535, default=8000, description="Port of the second FSSO collector agent.")    
    password2: Any = Field(max_length=128, default=None, description="Password of the second FSSO collector agent.")    
    server3: str | None = Field(max_length=63, default=None, description="Domain name or IP address of the third FSSO collector agent.")    
    port3: int | None = Field(ge=1, le=65535, default=8000, description="Port of the third FSSO collector agent.")    
    password3: Any = Field(max_length=128, default=None, description="Password of the third FSSO collector agent.")    
    server4: str | None = Field(max_length=63, default=None, description="Domain name or IP address of the fourth FSSO collector agent.")    
    port4: int | None = Field(ge=1, le=65535, default=8000, description="Port of the fourth FSSO collector agent.")    
    password4: Any = Field(max_length=128, default=None, description="Password of the fourth FSSO collector agent.")    
    server5: str | None = Field(max_length=63, default=None, description="Domain name or IP address of the fifth FSSO collector agent.")    
    port5: int | None = Field(ge=1, le=65535, default=8000, description="Port of the fifth FSSO collector agent.")    
    password5: Any = Field(max_length=128, default=None, description="Password of the fifth FSSO collector agent.")    
    logon_timeout: int | None = Field(ge=1, le=2880, default=5, description="Interval in minutes to keep logons after FSSO server down.")    
    ldap_server: str | None = Field(max_length=35, default=None, description="LDAP server to get group information.")  # datasource: ['user.ldap.name']    
    group_poll_interval: int | None = Field(ge=1, le=2880, default=0, description="Interval in minutes within to fetch groups from FSSO server, or unset to disable.")    
    ldap_poll: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable automatic fetching of groups from LDAP server.")    
    ldap_poll_interval: int | None = Field(ge=1, le=2880, default=180, description="Interval in minutes within to fetch groups from LDAP server.")    
    ldap_poll_filter: str | None = Field(max_length=2047, default="(objectCategory=group)", description="Filter used to fetch groups.")    
    user_info_server: str | None = Field(max_length=35, default=None, description="LDAP server to get user information.")  # datasource: ['user.ldap.name']    
    ssl: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable use of SSL.")    
    sni: str | None = Field(max_length=255, default=None, description="Server Name Indication.")    
    ssl_server_host_ip_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable server host/IP verification.")    
    ssl_trusted_cert: str | None = Field(max_length=79, default=None, description="Trusted server certificate or CA certificate.")  # datasource: ['vpn.certificate.remote.name', 'vpn.certificate.ca.name']    
    source_ip: str | None = Field(default="0.0.0.0", description="Source IP for communications to FSSO agent.")    
    source_ip6: str | None = Field(default="::", description="IPv6 source for communications to FSSO agent.")    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('ldap_server')
    @classmethod
    def validate_ldap_server(cls, v: Any) -> Any:
        """
        Validate ldap_server field.
        
        Datasource: ['user.ldap.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('user_info_server')
    @classmethod
    def validate_user_info_server(cls, v: Any) -> Any:
        """
        Validate user_info_server field.
        
        Datasource: ['user.ldap.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ssl_trusted_cert')
    @classmethod
    def validate_ssl_trusted_cert(cls, v: Any) -> Any:
        """
        Validate ssl_trusted_cert field.
        
        Datasource: ['vpn.certificate.remote.name', 'vpn.certificate.ca.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "FssoModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_ldap_server_references(self, client: Any) -> list[str]:
        """
        Validate ldap_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/ldap        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = FssoModel(
            ...     ldap_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ldap_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.fsso.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ldap_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.user.ldap.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ldap-Server '{value}' not found in "
                "user/ldap"
            )        
        return errors    
    async def validate_user_info_server_references(self, client: Any) -> list[str]:
        """
        Validate user_info_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/ldap        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = FssoModel(
            ...     user_info_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_user_info_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.fsso.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "user_info_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.user.ldap.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"User-Info-Server '{value}' not found in "
                "user/ldap"
            )        
        return errors    
    async def validate_ssl_trusted_cert_references(self, client: Any) -> list[str]:
        """
        Validate ssl_trusted_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/remote        - vpn/certificate/ca        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = FssoModel(
            ...     ssl_trusted_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssl_trusted_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.fsso.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ssl_trusted_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.remote.exists(value):
            found = True
        elif await client.api.cmdb.vpn.certificate.ca.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssl-Trusted-Cert '{value}' not found in "
                "vpn/certificate/remote or vpn/certificate/ca"
            )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = FssoModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.user.fsso.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_ldap_server_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_user_info_server_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ssl_trusted_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "FssoModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.042272Z
# ============================================================================


"""FortiOS CMDB - Time category"""

from ..time_base import Time as TimeBase
from .set import Set

__all__ = [
    "Set",
    "Time",
]


class Time(TimeBase):
    """Time endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Time endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.set = Set(client)



"""Validation helpers for user/setting - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "auth-type": "http https ftp telnet",
    "auth-cert": "",
    "auth-ca-cert": "",
    "auth-secure-http": "disable",
    "auth-http-basic": "disable",
    "auth-ssl-allow-renegotiation": "disable",
    "auth-src-mac": "enable",
    "auth-on-demand": "implicitly",
    "auth-timeout": 5,
    "auth-timeout-type": "idle-timeout",
    "auth-portal-timeout": 3,
    "radius-ses-timeout-act": "hard-timeout",
    "auth-blackout-time": 0,
    "auth-invalid-max": 5,
    "auth-lockout-threshold": 3,
    "auth-lockout-duration": 0,
    "per-policy-disclaimer": "disable",
    "auth-ssl-min-proto-version": "default",
    "auth-ssl-max-proto-version": "",
    "auth-ssl-sigalgs": "all",
    "default-user-password-policy": "",
    "cors": "disable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "auth-type": "option",  # Supported firewall policy authentication protocols/methods.
    "auth-cert": "string",  # HTTPS server certificate for policy authentication.
    "auth-ca-cert": "string",  # HTTPS CA certificate for policy authentication.
    "auth-secure-http": "option",  # Enable/disable redirecting HTTP user authentication to more 
    "auth-http-basic": "option",  # Enable/disable use of HTTP basic authentication for identity
    "auth-ssl-allow-renegotiation": "option",  # Allow/forbid SSL re-negotiation for HTTPS authentication.
    "auth-src-mac": "option",  # Enable/disable source MAC for user identity.
    "auth-on-demand": "option",  # Always/implicitly trigger firewall authentication on demand.
    "auth-timeout": "integer",  # Time in minutes before the firewall user authentication time
    "auth-timeout-type": "option",  # Control if authenticated users have to login again after a h
    "auth-portal-timeout": "integer",  # Time in minutes before captive portal user have to re-authen
    "radius-ses-timeout-act": "option",  # Set the RADIUS session timeout to a hard timeout or to ignor
    "auth-blackout-time": "integer",  # Time in seconds an IP address is denied access after failing
    "auth-invalid-max": "integer",  # Maximum number of failed authentication attempts before the 
    "auth-lockout-threshold": "integer",  # Maximum number of failed login attempts before login lockout
    "auth-lockout-duration": "integer",  # Lockout period in seconds after too many login failures.
    "per-policy-disclaimer": "option",  # Enable/disable per policy disclaimer.
    "auth-ports": "string",  # Set up non-standard ports for authentication with HTTP, HTTP
    "auth-ssl-min-proto-version": "option",  # Minimum supported protocol version for SSL/TLS connections (
    "auth-ssl-max-proto-version": "option",  # Maximum supported protocol version for SSL/TLS connections (
    "auth-ssl-sigalgs": "option",  # Set signature algorithms related to HTTPS authentication (af
    "default-user-password-policy": "string",  # Default password policy to apply to all local users unless o
    "cors": "option",  # Enable/disable allowed origins white list for CORS.
    "cors-allowed-origins": "string",  # Allowed origins white list for CORS.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "auth-type": "Supported firewall policy authentication protocols/methods.",
    "auth-cert": "HTTPS server certificate for policy authentication.",
    "auth-ca-cert": "HTTPS CA certificate for policy authentication.",
    "auth-secure-http": "Enable/disable redirecting HTTP user authentication to more secure HTTPS.",
    "auth-http-basic": "Enable/disable use of HTTP basic authentication for identity-based firewall policies.",
    "auth-ssl-allow-renegotiation": "Allow/forbid SSL re-negotiation for HTTPS authentication.",
    "auth-src-mac": "Enable/disable source MAC for user identity.",
    "auth-on-demand": "Always/implicitly trigger firewall authentication on demand.",
    "auth-timeout": "Time in minutes before the firewall user authentication timeout requires the user to re-authenticate.",
    "auth-timeout-type": "Control if authenticated users have to login again after a hard timeout, after an idle timeout, or after a session timeout.",
    "auth-portal-timeout": "Time in minutes before captive portal user have to re-authenticate (1 - 30 min, default 3 min).",
    "radius-ses-timeout-act": "Set the RADIUS session timeout to a hard timeout or to ignore RADIUS server session timeouts.",
    "auth-blackout-time": "Time in seconds an IP address is denied access after failing to authenticate five times within one minute.",
    "auth-invalid-max": "Maximum number of failed authentication attempts before the user is blocked.",
    "auth-lockout-threshold": "Maximum number of failed login attempts before login lockout is triggered.",
    "auth-lockout-duration": "Lockout period in seconds after too many login failures.",
    "per-policy-disclaimer": "Enable/disable per policy disclaimer.",
    "auth-ports": "Set up non-standard ports for authentication with HTTP, HTTPS, FTP, and TELNET.",
    "auth-ssl-min-proto-version": "Minimum supported protocol version for SSL/TLS connections (default is to follow system global setting).",
    "auth-ssl-max-proto-version": "Maximum supported protocol version for SSL/TLS connections (default is no limit).",
    "auth-ssl-sigalgs": "Set signature algorithms related to HTTPS authentication (affects TLS version <= 1.2 only, default is to enable all).",
    "default-user-password-policy": "Default password policy to apply to all local users unless otherwise specified, as defined in config user password-policy.",
    "cors": "Enable/disable allowed origins white list for CORS.",
    "cors-allowed-origins": "Allowed origins white list for CORS.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "auth-cert": {"type": "string", "max_length": 35},
    "auth-ca-cert": {"type": "string", "max_length": 35},
    "auth-timeout": {"type": "integer", "min": 1, "max": 1440},
    "auth-portal-timeout": {"type": "integer", "min": 1, "max": 30},
    "auth-blackout-time": {"type": "integer", "min": 0, "max": 3600},
    "auth-invalid-max": {"type": "integer", "min": 1, "max": 100},
    "auth-lockout-threshold": {"type": "integer", "min": 1, "max": 10},
    "auth-lockout-duration": {"type": "integer", "min": 0, "max": 4294967295},
    "default-user-password-policy": {"type": "string", "max_length": 35},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "auth-ports": {
        "id": {
            "type": "integer",
            "help": "ID.",
            "required": True,
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "type": {
            "type": "option",
            "help": "Service type.",
            "default": "http",
            "options": ["http", "https", "ftp", "telnet"],
        },
        "port": {
            "type": "integer",
            "help": "Non-standard port for firewall user authentication.",
            "default": 1024,
            "min_value": 1,
            "max_value": 65535,
        },
    },
    "cors-allowed-origins": {
        "name": {
            "type": "string",
            "help": "Allowed origin for CORS.",
            "default": "",
            "max_length": 79,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_AUTH_TYPE = [
    "http",
    "https",
    "ftp",
    "telnet",
]
VALID_BODY_AUTH_SECURE_HTTP = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_HTTP_BASIC = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_SSL_ALLOW_RENEGOTIATION = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_SRC_MAC = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_ON_DEMAND = [
    "always",
    "implicitly",
]
VALID_BODY_AUTH_TIMEOUT_TYPE = [
    "idle-timeout",
    "hard-timeout",
    "new-session",
]
VALID_BODY_RADIUS_SES_TIMEOUT_ACT = [
    "hard-timeout",
    "ignore-timeout",
]
VALID_BODY_PER_POLICY_DISCLAIMER = [
    "enable",
    "disable",
]
VALID_BODY_AUTH_SSL_MIN_PROTO_VERSION = [
    "default",
    "SSLv3",
    "TLSv1",
    "TLSv1-1",
    "TLSv1-2",
    "TLSv1-3",
]
VALID_BODY_AUTH_SSL_MAX_PROTO_VERSION = [
    "sslv3",
    "tlsv1",
    "tlsv1-1",
    "tlsv1-2",
    "tlsv1-3",
]
VALID_BODY_AUTH_SSL_SIGALGS = [
    "no-rsa-pss",
    "all",
]
VALID_BODY_CORS = [
    "disable",
    "enable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_user_setting_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for user/setting."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_user_setting_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new user/setting object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-type",
            payload["auth-type"],
            VALID_BODY_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-secure-http" in payload:
        is_valid, error = _validate_enum_field(
            "auth-secure-http",
            payload["auth-secure-http"],
            VALID_BODY_AUTH_SECURE_HTTP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-http-basic" in payload:
        is_valid, error = _validate_enum_field(
            "auth-http-basic",
            payload["auth-http-basic"],
            VALID_BODY_AUTH_HTTP_BASIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-allow-renegotiation" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-allow-renegotiation",
            payload["auth-ssl-allow-renegotiation"],
            VALID_BODY_AUTH_SSL_ALLOW_RENEGOTIATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-src-mac" in payload:
        is_valid, error = _validate_enum_field(
            "auth-src-mac",
            payload["auth-src-mac"],
            VALID_BODY_AUTH_SRC_MAC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-on-demand" in payload:
        is_valid, error = _validate_enum_field(
            "auth-on-demand",
            payload["auth-on-demand"],
            VALID_BODY_AUTH_ON_DEMAND,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-timeout-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-timeout-type",
            payload["auth-timeout-type"],
            VALID_BODY_AUTH_TIMEOUT_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "radius-ses-timeout-act" in payload:
        is_valid, error = _validate_enum_field(
            "radius-ses-timeout-act",
            payload["radius-ses-timeout-act"],
            VALID_BODY_RADIUS_SES_TIMEOUT_ACT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "per-policy-disclaimer" in payload:
        is_valid, error = _validate_enum_field(
            "per-policy-disclaimer",
            payload["per-policy-disclaimer"],
            VALID_BODY_PER_POLICY_DISCLAIMER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-min-proto-version",
            payload["auth-ssl-min-proto-version"],
            VALID_BODY_AUTH_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-max-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-max-proto-version",
            payload["auth-ssl-max-proto-version"],
            VALID_BODY_AUTH_SSL_MAX_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-sigalgs" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-sigalgs",
            payload["auth-ssl-sigalgs"],
            VALID_BODY_AUTH_SSL_SIGALGS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cors" in payload:
        is_valid, error = _validate_enum_field(
            "cors",
            payload["cors"],
            VALID_BODY_CORS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_user_setting_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update user/setting."""
    # Validate enum values using central function
    if "auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-type",
            payload["auth-type"],
            VALID_BODY_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-secure-http" in payload:
        is_valid, error = _validate_enum_field(
            "auth-secure-http",
            payload["auth-secure-http"],
            VALID_BODY_AUTH_SECURE_HTTP,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-http-basic" in payload:
        is_valid, error = _validate_enum_field(
            "auth-http-basic",
            payload["auth-http-basic"],
            VALID_BODY_AUTH_HTTP_BASIC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-allow-renegotiation" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-allow-renegotiation",
            payload["auth-ssl-allow-renegotiation"],
            VALID_BODY_AUTH_SSL_ALLOW_RENEGOTIATION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-src-mac" in payload:
        is_valid, error = _validate_enum_field(
            "auth-src-mac",
            payload["auth-src-mac"],
            VALID_BODY_AUTH_SRC_MAC,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-on-demand" in payload:
        is_valid, error = _validate_enum_field(
            "auth-on-demand",
            payload["auth-on-demand"],
            VALID_BODY_AUTH_ON_DEMAND,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-timeout-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-timeout-type",
            payload["auth-timeout-type"],
            VALID_BODY_AUTH_TIMEOUT_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "radius-ses-timeout-act" in payload:
        is_valid, error = _validate_enum_field(
            "radius-ses-timeout-act",
            payload["radius-ses-timeout-act"],
            VALID_BODY_RADIUS_SES_TIMEOUT_ACT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "per-policy-disclaimer" in payload:
        is_valid, error = _validate_enum_field(
            "per-policy-disclaimer",
            payload["per-policy-disclaimer"],
            VALID_BODY_PER_POLICY_DISCLAIMER,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-min-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-min-proto-version",
            payload["auth-ssl-min-proto-version"],
            VALID_BODY_AUTH_SSL_MIN_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-max-proto-version" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-max-proto-version",
            payload["auth-ssl-max-proto-version"],
            VALID_BODY_AUTH_SSL_MAX_PROTO_VERSION,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-ssl-sigalgs" in payload:
        is_valid, error = _validate_enum_field(
            "auth-ssl-sigalgs",
            payload["auth-ssl-sigalgs"],
            VALID_BODY_AUTH_SSL_SIGALGS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "cors" in payload:
        is_valid, error = _validate_enum_field(
            "cors",
            payload["cors"],
            VALID_BODY_CORS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "user/setting",
    "category": "cmdb",
    "api_path": "user/setting",
    "help": "Configure user authentication setting.",
    "total_fields": 24,
    "required_fields_count": 0,
    "fields_with_defaults_count": 22,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""FortiOS CMDB - WebProxy category"""

from .debug_url import DebugUrl
from .explicit import Explicit
from .fast_fallback import FastFallback
from .forward_server import ForwardServer
from .forward_server_group import ForwardServerGroup
from .global_ import Global
from .isolator_server import IsolatorServer
from .profile import Profile
from .url_match import UrlMatch
from .wisp import Wisp

__all__ = [
    "DebugUrl",
    "Explicit",
    "FastFallback",
    "ForwardServer",
    "ForwardServerGroup",
    "Global",
    "IsolatorServer",
    "Profile",
    "UrlMatch",
    "WebProxy",
    "Wisp",
]


class WebProxy:
    """WebProxy endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """WebProxy endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.debug_url = DebugUrl(client)
        self.explicit = Explicit(client)
        self.fast_fallback = FastFallback(client)
        self.forward_server = ForwardServer(client)
        self.forward_server_group = ForwardServerGroup(client)
        self.global_ = Global(client)
        self.isolator_server = IsolatorServer(client)
        self.profile = Profile(client)
        self.url_match = UrlMatch(client)
        self.wisp = Wisp(client)



"""
Pydantic Models for CMDB - ztna/web_portal

Runtime validation models for ztna/web_portal configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class WebPortalThemeEnum(str, Enum):
    """Allowed values for theme field."""
    JADE = "jade"
    NEUTRINO = "neutrino"
    MARINER = "mariner"
    GRAPHITE = "graphite"
    MELONGENE = "melongene"
    JET_STREAM = "jet-stream"
    SECURITY_FABRIC = "security-fabric"
    DARK_MATTER = "dark-matter"
    ONYX = "onyx"
    ECLIPSE = "eclipse"


# ============================================================================
# Main Model
# ============================================================================

class WebPortalModel(BaseModel):
    """
    Pydantic model for ztna/web_portal configuration.
    
    Configure ztna web-portal.
    
    Validation Rules:        - name: max_length=79 pattern=        - vip: max_length=79 pattern=        - host: max_length=79 pattern=        - decrypted_traffic_mirror: max_length=35 pattern=        - log_blocked_traffic: pattern=        - auth_portal: pattern=        - auth_virtual_host: max_length=79 pattern=        - vip6: max_length=79 pattern=        - auth_rule: max_length=35 pattern=        - display_bookmark: pattern=        - focus_bookmark: pattern=        - display_status: pattern=        - display_history: pattern=        - policy_auth_sso: pattern=        - heading: max_length=31 pattern=        - theme: pattern=        - clipboard: pattern=        - default_window_width: min=0 max=65535 pattern=        - default_window_height: min=0 max=65535 pattern=        - cookie_age: min=0 max=525600 pattern=        - forticlient_download: pattern=        - customize_forticlient_download_url: pattern=        - windows_forticlient_download_url: max_length=1023 pattern=        - macos_forticlient_download_url: max_length=1023 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=79, default=None, description="ZTNA proxy name.")    
    vip: str | None = Field(max_length=79, default=None, description="Virtual IP name.")  # datasource: ['firewall.vip.name']    
    host: str | None = Field(max_length=79, default=None, description="Virtual or real host name.")  # datasource: ['firewall.access-proxy-virtual-host.name']    
    decrypted_traffic_mirror: str | None = Field(max_length=35, default=None, description="Decrypted traffic mirror.")  # datasource: ['firewall.decrypted-traffic-mirror.name']    
    log_blocked_traffic: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable logging of blocked traffic.")    
    auth_portal: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable authentication portal.")    
    auth_virtual_host: str | None = Field(max_length=79, default=None, description="Virtual host for authentication portal.")  # datasource: ['firewall.access-proxy-virtual-host.name']    
    vip6: str | None = Field(max_length=79, default=None, description="Virtual IPv6 name.")  # datasource: ['firewall.vip6.name']    
    auth_rule: str | None = Field(max_length=35, default=None, description="Authentication Rule.")  # datasource: ['authentication.rule.name']    
    display_bookmark: Literal["enable", "disable"] | None = Field(default="enable", description="Enable to display the web portal bookmark widget.")    
    focus_bookmark: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to prioritize the placement of the bookmark section over the quick-connection section in the ztna web-portal.")    
    display_status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable to display the web portal status widget.")    
    display_history: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to display the web portal user login history widget.")    
    policy_auth_sso: Literal["enable", "disable"] | None = Field(default="enable", description="Enable policy sso authentication.")    
    heading: str | None = Field(max_length=31, default="ZTNA Portal", description="Web portal heading message.")    
    theme: WebPortalThemeEnum | None = Field(default=WebPortalThemeEnum.SECURITY_FABRIC, description="Web portal color scheme.")    
    clipboard: Literal["enable", "disable"] | None = Field(default="enable", description="Enable to support RDP/VPC clipboard functionality.")    
    default_window_width: int | None = Field(ge=0, le=65535, default=1024, description="Screen width (range from 0 - 65535, default = 1024).")    
    default_window_height: int | None = Field(ge=0, le=65535, default=768, description="Screen height (range from 0 - 65535, default = 768).")    
    cookie_age: int | None = Field(ge=0, le=525600, default=60, description="Time in minutes that client web browsers should keep a cookie. Default is 60 minutes. 0 = no time limit.")    
    forticlient_download: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable download option for FortiClient.")    
    customize_forticlient_download_url: Literal["enable", "disable"] | None = Field(default="disable", description="Enable support of customized download URL for FortiClient.")    
    windows_forticlient_download_url: str | None = Field(max_length=1023, default=None, description="Download URL for Windows FortiClient.")    
    macos_forticlient_download_url: str | None = Field(max_length=1023, default=None, description="Download URL for Mac FortiClient.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('vip')
    @classmethod
    def validate_vip(cls, v: Any) -> Any:
        """
        Validate vip field.
        
        Datasource: ['firewall.vip.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('host')
    @classmethod
    def validate_host(cls, v: Any) -> Any:
        """
        Validate host field.
        
        Datasource: ['firewall.access-proxy-virtual-host.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('decrypted_traffic_mirror')
    @classmethod
    def validate_decrypted_traffic_mirror(cls, v: Any) -> Any:
        """
        Validate decrypted_traffic_mirror field.
        
        Datasource: ['firewall.decrypted-traffic-mirror.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('auth_virtual_host')
    @classmethod
    def validate_auth_virtual_host(cls, v: Any) -> Any:
        """
        Validate auth_virtual_host field.
        
        Datasource: ['firewall.access-proxy-virtual-host.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('vip6')
    @classmethod
    def validate_vip6(cls, v: Any) -> Any:
        """
        Validate vip6 field.
        
        Datasource: ['firewall.vip6.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('auth_rule')
    @classmethod
    def validate_auth_rule(cls, v: Any) -> Any:
        """
        Validate auth_rule field.
        
        Datasource: ['authentication.rule.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "WebPortalModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_vip_references(self, client: Any) -> list[str]:
        """
        Validate vip references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/vip        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WebPortalModel(
            ...     vip="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vip_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ztna.web_portal.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "vip", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.vip.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Vip '{value}' not found in "
                "firewall/vip"
            )        
        return errors    
    async def validate_host_references(self, client: Any) -> list[str]:
        """
        Validate host references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/access-proxy-virtual-host        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WebPortalModel(
            ...     host="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_host_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ztna.web_portal.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "host", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.access_proxy_virtual_host.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Host '{value}' not found in "
                "firewall/access-proxy-virtual-host"
            )        
        return errors    
    async def validate_decrypted_traffic_mirror_references(self, client: Any) -> list[str]:
        """
        Validate decrypted_traffic_mirror references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/decrypted-traffic-mirror        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WebPortalModel(
            ...     decrypted_traffic_mirror="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_decrypted_traffic_mirror_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ztna.web_portal.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "decrypted_traffic_mirror", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.decrypted_traffic_mirror.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Decrypted-Traffic-Mirror '{value}' not found in "
                "firewall/decrypted-traffic-mirror"
            )        
        return errors    
    async def validate_auth_virtual_host_references(self, client: Any) -> list[str]:
        """
        Validate auth_virtual_host references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/access-proxy-virtual-host        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WebPortalModel(
            ...     auth_virtual_host="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_auth_virtual_host_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ztna.web_portal.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "auth_virtual_host", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.access_proxy_virtual_host.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Auth-Virtual-Host '{value}' not found in "
                "firewall/access-proxy-virtual-host"
            )        
        return errors    
    async def validate_vip6_references(self, client: Any) -> list[str]:
        """
        Validate vip6 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/vip6        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WebPortalModel(
            ...     vip6="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vip6_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ztna.web_portal.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "vip6", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.vip6.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Vip6 '{value}' not found in "
                "firewall/vip6"
            )        
        return errors    
    async def validate_auth_rule_references(self, client: Any) -> list[str]:
        """
        Validate auth_rule references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - authentication/rule        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = WebPortalModel(
            ...     auth_rule="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_auth_rule_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.ztna.web_portal.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "auth_rule", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.authentication.rule.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Auth-Rule '{value}' not found in "
                "authentication/rule"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_vip_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_host_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_decrypted_traffic_mirror_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_auth_virtual_host_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_vip6_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_auth_rule_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "WebPortalModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:14.271347Z
# ============================================================================


"""Type stubs for DLP category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .data_type import DataType
    from .dictionary import Dictionary
    from .exact_data_match import ExactDataMatch
    from .filepattern import Filepattern
    from .label import Label
    from .profile import Profile
    from .sensor import Sensor
    from .settings import Settings

__all__ = [
    "DataType",
    "Dictionary",
    "ExactDataMatch",
    "Filepattern",
    "Label",
    "Profile",
    "Sensor",
    "Settings",
    "Dlp",
]


class Dlp:
    """DLP API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    data_type: DataType
    dictionary: Dictionary
    exact_data_match: ExactDataMatch
    filepattern: Filepattern
    label: Label
    profile: Profile
    sensor: Sensor
    settings: Settings

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize dlp category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - webfilter/search_engine

Runtime validation models for webfilter/search_engine configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SearchEngineSafesearchEnum(str, Enum):
    """Allowed values for safesearch field."""
    DISABLE = "disable"
    URL = "url"
    HEADER = "header"
    TRANSLATE = "translate"
    YT_PATTERN = "yt-pattern"
    YT_SCAN = "yt-scan"
    YT_VIDEO = "yt-video"
    YT_CHANNEL = "yt-channel"


# ============================================================================
# Main Model
# ============================================================================

class SearchEngineModel(BaseModel):
    """
    Pydantic model for webfilter/search_engine configuration.
    
    Configure web filter search engines.
    
    Validation Rules:        - name: max_length=35 pattern=        - hostname: max_length=127 pattern=        - url: max_length=127 pattern=        - query: max_length=15 pattern=        - safesearch: pattern=        - charset: pattern=        - safesearch_str: max_length=255 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=35, description="Search engine name.")    
    hostname: str | None = Field(max_length=127, default=None, description="Hostname (regular expression).")    
    url: str | None = Field(max_length=127, default=None, description="URL (regular expression).")    
    query: str | None = Field(max_length=15, default=None, description="Code used to prefix a query (must end with an equals character).")    
    safesearch: SearchEngineSafesearchEnum | None = Field(default=SearchEngineSafesearchEnum.DISABLE, description="Safe search method. You can disable safe search, add the safe search string to URLs, or insert a safe search header.")    
    charset: Literal["utf-8", "gb2312"] | None = Field(default="utf-8", description="Search engine charset.")    
    safesearch_str: str | None = Field(max_length=255, default=None, description="Safe search parameter used in the URL in URL mode. In translate mode, it provides either the regex to translate the URL or the special case to translate the URL.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SearchEngineModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SearchEngineModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.990955Z
# ============================================================================


"""
Pydantic Models for CMDB - vpn/ipsec/fec

Runtime validation models for vpn/ipsec/fec configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class FecMappings(BaseModel):
    """
    Child table model for mappings.
    
    FEC redundancy mapping table.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    seqno: int = Field(ge=0, le=64, default=0, description="Sequence number (1 - 64).")    
    base: int = Field(ge=1, le=20, default=0, description="Number of base FEC packets (1 - 20).")    
    redundant: int = Field(ge=1, le=5, default=0, description="Number of redundant FEC packets (1 - 5).")    
    packet_loss_threshold: int | None = Field(ge=0, le=100, default=0, description="Apply FEC parameters when packet loss is >= threshold (0 - 100, 0 means no threshold).")    
    latency_threshold: int | None = Field(ge=0, le=4294967295, default=0, description="Apply FEC parameters when latency is <= threshold (0 means no threshold).")    
    bandwidth_up_threshold: int | None = Field(ge=0, le=4294967295, default=0, description="Apply FEC parameters when available up bandwidth is >= threshold (kbps, 0 means no threshold).")    
    bandwidth_down_threshold: int | None = Field(ge=0, le=4294967295, default=0, description="Apply FEC parameters when available down bandwidth is >= threshold (kbps, 0 means no threshold).")    
    bandwidth_bi_threshold: int | None = Field(ge=0, le=4294967295, default=0, description="Apply FEC parameters when available bi-bandwidth is >= threshold (kbps, 0 means no threshold).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class FecModel(BaseModel):
    """
    Pydantic model for vpn/ipsec/fec configuration.
    
    Configure Forward Error Correction (FEC) mapping profiles.
    
    Validation Rules:        - name: max_length=35 pattern=        - mappings: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=35, description="Profile name.")    
    mappings: list[FecMappings] = Field(description="FEC redundancy mapping table.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "FecModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "FecModel",    "FecMappings",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.859637Z
# ============================================================================


"""FortiOS CMDB - Interface category"""

from ..interface_base import Interface as InterfaceBase
from .dhcp_renew import DhcpRenew
from .dhcp_status import DhcpStatus
from .kernel_interfaces import KernelInterfaces
from .poe import Poe
from .poe_usage import PoeUsage
from .speed_test_status import SpeedTestStatus
from .speed_test_trigger import SpeedTestTrigger
from .transceivers import Transceivers
from .wake_on_lan import WakeOnLan

__all__ = [
    "DhcpRenew",
    "DhcpStatus",
    "Interface",
    "KernelInterfaces",
    "Poe",
    "PoeUsage",
    "SpeedTestStatus",
    "SpeedTestTrigger",
    "Transceivers",
    "WakeOnLan",
]


class Interface(InterfaceBase):
    """Interface endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Interface endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.dhcp_renew = DhcpRenew(client)
        self.dhcp_status = DhcpStatus(client)
        self.kernel_interfaces = KernelInterfaces(client)
        self.poe = Poe(client)
        self.poe_usage = PoeUsage(client)
        self.speed_test_status = SpeedTestStatus(client)
        self.speed_test_trigger = SpeedTestTrigger(client)
        self.transceivers = Transceivers(client)
        self.wake_on_lan = WakeOnLan(client)



"""FortiOS CMDB - EthernetOam category"""

from .cfm import Cfm

__all__ = [
    "Cfm",
    "EthernetOam",
]


class EthernetOam:
    """EthernetOam endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """EthernetOam endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.cfm = Cfm(client)



"""Type stubs for PROXY category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .sessions import Sessions

__all__ = [
    "Sessions",
    "Proxy",
]


class Proxy:
    """PROXY API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    sessions: Sessions

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize proxy category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - switch_controller/snmp_community

Runtime validation models for switch_controller/snmp_community configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SnmpCommunityHosts(BaseModel):
    """
    Child table model for hosts.
    
    Configure IPv4 SNMP managers (hosts).
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Host entry ID.")    
    ip: str = Field(description="IPv4 address of the SNMP manager (host).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SnmpCommunityEventsEnum(str, Enum):
    """Allowed values for events field."""
    CPU_HIGH = "cpu-high"
    MEM_LOW = "mem-low"
    LOG_FULL = "log-full"
    INTF_IP = "intf-ip"
    ENT_CONF_CHANGE = "ent-conf-change"
    L2MAC = "l2mac"


# ============================================================================
# Main Model
# ============================================================================

class SnmpCommunityModel(BaseModel):
    """
    Pydantic model for switch_controller/snmp_community configuration.
    
    Configure FortiSwitch SNMP v1/v2c communities globally.
    
    Validation Rules:        - id_: min=0 max=4294967295 pattern=        - name: max_length=35 pattern=        - status: pattern=        - hosts: pattern=        - query_v1_status: pattern=        - query_v1_port: min=0 max=65535 pattern=        - query_v2c_status: pattern=        - query_v2c_port: min=0 max=65535 pattern=        - trap_v1_status: pattern=        - trap_v1_lport: min=0 max=65535 pattern=        - trap_v1_rport: min=0 max=65535 pattern=        - trap_v2c_status: pattern=        - trap_v2c_lport: min=0 max=65535 pattern=        - trap_v2c_rport: min=0 max=65535 pattern=        - events: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="SNMP community ID.")    
    name: str = Field(max_length=35, description="SNMP community name.")    
    status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable this SNMP community.")    
    hosts: list[SnmpCommunityHosts] = Field(default_factory=list, description="Configure IPv4 SNMP managers (hosts).")    
    query_v1_status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable SNMP v1 queries.")    
    query_v1_port: int | None = Field(ge=0, le=65535, default=161, description="SNMP v1 query port (default = 161).")    
    query_v2c_status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable SNMP v2c queries.")    
    query_v2c_port: int | None = Field(ge=0, le=65535, default=161, description="SNMP v2c query port (default = 161).")    
    trap_v1_status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable SNMP v1 traps.")    
    trap_v1_lport: int | None = Field(ge=0, le=65535, default=162, description="SNMP v2c trap local port (default = 162).")    
    trap_v1_rport: int | None = Field(ge=0, le=65535, default=162, description="SNMP v2c trap remote port (default = 162).")    
    trap_v2c_status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable SNMP v2c traps.")    
    trap_v2c_lport: int | None = Field(ge=0, le=65535, default=162, description="SNMP v2c trap local port (default = 162).")    
    trap_v2c_rport: int | None = Field(ge=0, le=65535, default=162, description="SNMP v2c trap remote port (default = 162).")    
    events: list[SnmpCommunityEventsEnum] = Field(default_factory=list, description="SNMP notifications (traps) to send.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SnmpCommunityModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SnmpCommunityModel",    "SnmpCommunityHosts",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.745404Z
# ============================================================================


"""Type stubs for FORTIGUARD category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .clear_statistics import ClearStatistics
    from .manual_update import ManualUpdate
    from .server_info import ServerInfo
    from .test_availability import TestAvailability
    from .update import Update

__all__ = [
    "ClearStatistics",
    "ManualUpdate",
    "ServerInfo",
    "TestAvailability",
    "Update",
    "Fortiguard",
]


class Fortiguard:
    """FORTIGUARD API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    clear_statistics: ClearStatistics
    manual_update: ManualUpdate
    server_info: ServerInfo
    test_availability: TestAvailability
    update: Update

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiguard category with HTTP client."""
        ...



"""
Pydantic Models for CMDB - dnsfilter/domain_filter

Runtime validation models for dnsfilter/domain_filter configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class DomainFilterEntries(BaseModel):
    """
    Child table model for entries.
    
    DNS domain filter entries.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Id.")    
    domain: str | None = Field(max_length=511, default=None, description="Domain entries to be filtered.")    
    type_: Literal["simple", "regex", "wildcard"] | None = Field(default="simple", serialization_alias="type", description="DNS domain filter type.")    
    action: Literal["block", "allow", "monitor"] | None = Field(default="block", description="Action to take for domain filter matches.")    
    status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable this domain filter.")    
    comment: str | None = Field(max_length=255, default=None, description="Comment.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class DomainFilterModel(BaseModel):
    """
    Pydantic model for dnsfilter/domain_filter configuration.
    
    Configure DNS domain filters.
    
    Validation Rules:        - id_: min=0 max=4294967295 pattern=        - name: max_length=63 pattern=        - comment: max_length=255 pattern=        - entries: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    name: str = Field(max_length=63, description="Name of table.")    
    comment: str | None = Field(max_length=255, default=None, description="Optional comments.")    
    entries: list[DomainFilterEntries] = Field(description="DNS domain filter entries.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "DomainFilterModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "DomainFilterModel",    "DomainFilterEntries",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.749644Z
# ============================================================================


"""Type stubs for NETWORK category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .arp import Arp
    from .reverse_ip_lookup import ReverseIpLookup
    from .ddns import Ddns
    from .debug_flow import DebugFlow
    from .dns import Dns
    from .fortiguard import Fortiguard
    from .lldp import Lldp

__all__ = [
    "Arp",
    "ReverseIpLookup",
    "Network",
]


class Network:
    """NETWORK API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    ddns: Ddns
    debug_flow: DebugFlow
    dns: Dns
    fortiguard: Fortiguard
    lldp: Lldp
    arp: Arp
    reverse_ip_lookup: ReverseIpLookup

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize network category with HTTP client."""
        ...



"""FortiOS CMDB - SdnConnector category"""

from .nsx_security_tags import NsxSecurityTags
from .status import Status
from .update import Update
from .validate_gcp_key import ValidateGcpKey

__all__ = [
    "NsxSecurityTags",
    "SdnConnector",
    "Status",
    "Update",
    "ValidateGcpKey",
]


class SdnConnector:
    """SdnConnector endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """SdnConnector endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.nsx_security_tags = NsxSecurityTags(client)
        self.status = Status(client)
        self.update = Update(client)
        self.validate_gcp_key = ValidateGcpKey(client)
